#include <avr\io.h>

#define LCD_DATA_PORT	PORTC
#define LCD_DATA_DIR	DDRC
#define LCD_CTRL_PORT	PORTA
#define LCD_CTRL_DIR	DDRA
#define LCD_RS_PIN		7
#define LCD_EN_PIN		6
#define LCD_RW_PIN		5
#define CLR_SCR			0x01
#define F_CPU			16000000

#include <util\delay.h>
#include <string.h>
#include <stdlib.h>
#include <math.h>

void check_busy(void);
void send_character(char c);
void send_cmd(unsigned char cmd);
void enable(void);
void peek(void);
void send_string(char* str);
void draw_menu(int i);
void set_selector(char oldpoz, int newpoz);
void switch_selection(void);
void update_parameter(int increase_or_decrease);
void start_adc(void);
void start_pwm(void);
uint16_t sample(void);



int selector_position = 0;
int b1 = 0;
int b2 = 0;
int b3 = 0;
int b4 = 0;
int current_menu = 1;
int current_fbk = 25;
int current_time = 300;
char str[4];


uint8_t buffer[1500];

int main(void)
{
int i = 0, j = 1;
uint16_t temp;
int amt = 4;

LCD_CTRL_DIR |= (1<<LCD_EN_PIN) | (1<<LCD_RW_PIN) | (1<<LCD_RS_PIN);
_delay_ms(15);

send_cmd(CLR_SCR);
_delay_ms(2);

send_cmd(0x38); //Set to 8 bit mode;
_delay_us(50);

send_cmd(0b00001100);
_delay_us(50);

send_cmd(0x02); //Reset cursor to initial position.
_delay_us(50);


draw_menu(1);
_delay_ms(15);

start_adc();
DDRD |= (1 << PD7);		
TCCR2A |= (1 << COM2A1) | (1 << WGM21) | (1 << WGM20);
TCCR2B |= (1 << CS20);

while(1)
{
	temp = sample() + buffer[j]*amt;
	temp = temp/8;
	if(temp > 255)
		temp = 255;
	_delay_us(50);
	buffer[i] = temp;
	OCR2A = temp;

	if(!(PIND & (1<<PD1)))
	{
		if(amt < 4)
			amt++;
		_delay_ms(300);
	}
	else if(!(PIND & (1<<PD2)))
	{
		if(amt > 0)
			amt--;
		_delay_ms(300);
	}
	i++;
	if(i == 1499)
		i = 0;
	j++;
	if(j == 1499)
		j = 0;
}

return 0;
}

void check_busy()
{
LCD_DATA_DIR = 0;

LCD_CTRL_PORT |= (1<<LCD_RW_PIN);
LCD_CTRL_PORT &= ~(1<<LCD_RS_PIN);

while(LCD_DATA_PORT >= 0x80)
{;}

LCD_DATA_DIR = 0xFF;
}


void send_character(char c)
{
check_busy();

LCD_DATA_PORT = c;
LCD_CTRL_PORT &= ~(1<<LCD_RW_PIN);
LCD_CTRL_PORT |= (1<<LCD_RS_PIN);

peek();

LCD_DATA_PORT = 0;

}

void send_cmd(unsigned char cmd)
{
check_busy();

LCD_DATA_PORT = cmd;
LCD_CTRL_PORT &= ~(1<<LCD_RW_PIN);
LCD_CTRL_PORT &= ~(1<<LCD_RS_PIN);

peek();

LCD_DATA_PORT = 0;

}

void peek()
{
LCD_CTRL_PORT |= (1<<LCD_EN_PIN);

asm volatile ("nop");
asm volatile ("nop");

LCD_CTRL_PORT &= ~(1<<LCD_EN_PIN);
}


void send_string(char* str)
{
while(*str != 0)
{
	send_character(*str++);
	_delay_ms(15);
}

}


void draw_menu(int i)
{
selector_position = 0;
switch(i)
{
case 1:
	current_fbk = 25;
	current_time = 300;
	send_cmd(0x80);
	_delay_us(50);
	send_cmd(0x88);
	_delay_us(50);
	send_string("Delay");
	_delay_ms(15);
	send_cmd(0xC0);
	_delay_us(50);
	send_character(0x3E); //sageata selectie
	_delay_ms(15);
	send_string("Time:300");
	_delay_ms(15);
	send_cmd(0xCE);
	_delay_us(50);
	send_string("Fbk:25");
	_delay_ms(15);
	break;
case 2:
	current_fbk = 25;
	current_time = 30;
	send_cmd(0x80);
	_delay_us(50);
	send_cmd(0x87);
	_delay_us(50);
	send_character(0xEA);
	_delay_ms(15);
	send_string("Delay");
	_delay_ms(15);
	send_cmd(0xC0);
	_delay_us(50);
	send_character(0x3E); //sageata selectie
	_delay_ms(15);
	send_string("Time:30");
	_delay_ms(15);
	send_cmd(0xCE);
	_delay_us(50);
	send_string("Fbk:25");
	_delay_ms(15);
	break;
case 3:
	current_fbk = 25;
	current_time = 30;
	send_cmd(0x80);
	_delay_us(50);
	send_cmd(0x87);
	_delay_us(50);
	send_string("Chorus");
	_delay_ms(15);
	send_cmd(0xC0);
	_delay_us(50);
	send_character(0x3E); //sageata selectie
	_delay_ms(15);
	send_string("Rate:30");
	_delay_ms(15);
	send_cmd(0xCE);
	_delay_us(50);
	send_string("Fbk:25");
	_delay_ms(15);
	break;
default:
	break;
}
}

void set_selector(char oldpoz, int newpoz)
{
send_cmd(oldpoz);
_delay_us(50);
send_character(' ');
_delay_ms(15);
send_cmd(newpoz);
_delay_us(50);
send_character(0x3E);
_delay_ms(15);
}

void switch_selection()
{
	if(!selector_position)
		{
		set_selector(0xC0, 0xCD);
		selector_position = 1;
		}
	else
		{
		set_selector(0xCD, 0xC0);
		selector_position = 0;
		}

}


void update_parameter(int increase_or_decrease)
{
if(increase_or_decrease == 0) //increase;
{
	if(selector_position == 0)
	{
		current_time++;
		send_cmd(0xC6);
		_delay_us(50);
		memset(str, 0, 4);
		itoa(current_time, str, 10);
		send_string(str);
		_delay_ms(15);
	}
	else
	{
		current_fbk++;
		send_cmd(0xD2);
		_delay_us(50);
		memset(str, 0, 4);
		itoa(current_fbk, str, 10);
		send_string(str);
		_delay_ms(15);
	}
}
else
	{
	if(selector_position == 0)
	{
		current_time--;
		send_cmd(0xC6);
		_delay_us(50);
		memset(str, 0, 4);
		itoa(current_time, str, 10);
		send_string(str);
		_delay_ms(15);
	}
	else
	{
		current_fbk--;
		send_cmd(0xD2);
		_delay_us(50);
		memset(str, 0, 4);
		itoa(current_fbk, str, 10);
		send_string(str);
		_delay_ms(15);
	}
}

}

void start_adc()
{
ADMUX = 0b11000000;
ADCSRA = 0b10000010;
}

void start_pwm()
{
TCCR1B |= (1<<WGM10) | (1<<WGM11) | (1<<COM1B0) | (1<<COM1B1);
DDRD |= (1<<PD4);
}

uint16_t sample()
{
uint16_t sampL, sampH;

ADCSRA |= 1<<ADSC;
while(!ADIF) //wait for the conversion to finish and generate an interrupt;
ADCSRA |= 1<<ADIF;
sampL = ADCL;
sampH = ADCH;
sampH = sampH<<8; //shift the high part of the sample;
sampH |= sampL; //compute the large sample;
return sampH;
}

/**
LCD Assignments:
1 - VSS <-> GND
2 - VDD <-> VCC
3 - V0	<-> POT
4 - RS	<-> PA7
5 - R/W <-> PA5
6 - EN	<-> PA6
DATA0-7 <-> PC0-7
15 A	<-> VCC
16 K	<-> GND

LCD DDRAM Addr:
Row1: 0x80 - 0x93
Row2: 0xC0 - 0xD3
*/